@@ -218,6 +218,7 @@ function configure_trove {
     iniset_conditional $TROVE_CONF DEFAULT usage_timeout $TROVE_USAGE_TIMEOUT
     iniset_conditional $TROVE_CONF DEFAULT state_change_wait_time $TROVE_STATE_CHANGE_WAIT_TIME
     iniset_conditional $TROVE_CONF DEFAULT reboot_time_out 300
+    iniset $TROVE_CONF DEFAULT controller_address ${SERVICE_HOST}
 
     configure_keystone_authtoken_middleware $TROVE_CONF trove
     iniset $TROVE_CONF service_credentials username trove
@@ -362,7 +363,7 @@ function create_subnet_v6 {
     if [[ -n "$IPV6_PRIVATE_NETWORK_GATEWAY" ]]; then
         subnet_params+="--gateway $IPV6_PRIVATE_NETWORK_GATEWAY "
     fi
-    if [ -n $SUBNETPOOL_V6_ID ]; then
+    if [[ -n $SUBNETPOOL_V6_ID ]]; then
         subnet_params+="--subnet-pool $SUBNETPOOL_V6_ID "
     else
         subnet_params+="--subnet-range $FIXED_RANGE_V6 $ipv6_modes} "
@@ -447,27 +448,26 @@ function create_guest_image {
         return 0
     fi
 
-    image_name="trove-datastore-${TROVE_IMAGE_OS}-${TROVE_IMAGE_OS_RELEASE}-${TROVE_DATASTORE_TYPE}"
-    image_url_var="TROVE_NON_DEV_IMAGE_URL_${TROVE_DATASTORE_TYPE^^}"
-    image_url=`eval echo '$'"$image_url_var"`
+    image_name="trove-guest-${TROVE_IMAGE_OS}-${TROVE_IMAGE_OS_RELEASE}"
     mkdir -p $HOME/images
     image_file=$HOME/images/${image_name}.qcow2
 
-    if [[ -n ${image_url} ]]; then
-        echo "Downloading guest image from ${image_url}"
-        curl -sSL ${image_url} -o ${image_file}
+    if [[ -n ${TROVE_NON_DEV_IMAGE_URL} ]]; then
+        echo "Downloading guest image from ${TROVE_NON_DEV_IMAGE_URL}"
+        curl -sSL ${TROVE_NON_DEV_IMAGE_URL} -o ${image_file}
     else
         echo "Starting to create guest image"
 
-        TROVE_BRANCH=${TROVE_BRANCH} $DEST/trove/integration/scripts/trovestack \
+        $DEST/trove/integration/scripts/trovestack \
           build-image \
-          ${TROVE_DATASTORE_TYPE} \
           ${TROVE_IMAGE_OS} \
           ${TROVE_IMAGE_OS_RELEASE} \
-          true
+          true \
+          ${TROVE_IMAGE_OS} \
+          ${image_file}
     fi
 
-    if [ ! -f ${image_file} ]; then
+    if [[ ! -f ${image_file} ]]; then
         echo "Image file was not found at ${image_file}"
         exit 1
     fi
@@ -485,7 +485,7 @@ function create_guest_image {
     $TROVE_MANAGE datastore_update $TROVE_DATASTORE_TYPE $TROVE_DATASTORE_VERSION
 
     echo "Add parameter validation rules if available"
-    if [ -f $DEST/trove/trove/templates/$TROVE_DATASTORE_TYPE/validation-rules.json ]; then
+    if [[ -f $DEST/trove/trove/templates/$TROVE_DATASTORE_TYPE/validation-rules.json ]]; then
         $TROVE_MANAGE db_load_datastore_config_parameters "$TROVE_DATASTORE_TYPE" "$TROVE_DATASTORE_VERSION" \
             $DEST/trove/trove/templates/$TROVE_DATASTORE_TYPE/validation-rules.json
     fi
@@ -556,7 +556,6 @@ function config_trove_network {
     sudo ip route
 
     # Now make sure the conf settings are right
-    iniset $TROVE_CONF DEFAULT network_label_regex ""
     iniset $TROVE_CONF DEFAULT ip_regex ""
     iniset $TROVE_CONF DEFAULT black_list_regex ""
     iniset $TROVE_CONF DEFAULT management_networks ${mgmt_net_id}
