@@ -25,7 +25,6 @@ import traceback
 import eventlet
 from oslo_log import log as logging
 import proboscis
-import six
 from six.moves import urllib
 import wsgi_intercept
 from wsgi_intercept.httplib2_intercept import install as wsgi_install
@@ -57,10 +56,7 @@ def add_support_for_localization():
     if os.path.exists(os.path.join(possible_topdir, 'nova', '__init__.py')):
         sys.path.insert(0, possible_topdir)
 
-    if six.PY3:
-        gettext.install('nova')
-    else:
-        gettext.install('nova', unicode=True)
+    gettext.install('nova')
 
 
 def initialize_trove(config_file):
@@ -206,24 +202,14 @@ def import_tests():
     from trove.tests.api import configurations  # noqa
     from trove.tests.api import databases  # noqa
     from trove.tests.api import datastores  # noqa
-    from trove.tests.api import flavors  # noqa
-    from trove.tests.api import header  # noqa
     from trove.tests.api import instances as rd_instances  # noqa
     from trove.tests.api import instances_actions as rd_actions  # noqa
     from trove.tests.api import instances_delete  # noqa
-    from trove.tests.api import instances_mysql_down  # noqa
     from trove.tests.api import instances_resize  # noqa
     from trove.tests.api import limits  # noqa
-    from trove.tests.api.mgmt import accounts  # noqa
-    from trove.tests.api.mgmt import admin_required  # noqa
-    from trove.tests.api.mgmt import hosts  # noqa
-    from trove.tests.api.mgmt import instances as mgmt_instances  # noqa
     from trove.tests.api.mgmt import instances_actions as mgmt_actions  # noqa
-    from trove.tests.api.mgmt import malformed_json  # noqa
-    from trove.tests.api.mgmt import storage  # noqa
     from trove.tests.api import replication  # noqa
     from trove.tests.api import root  # noqa
-    from trove.tests.api import root_on_create  # noqa
     from trove.tests.api import user_access  # noqa
     from trove.tests.api import users  # noqa
     from trove.tests.api import versions  # noqa
