@@ -80,12 +80,19 @@ class Commands(object):
 
     def db_load_datastore_config_parameters(self,
                                             datastore,
-                                            datastore_version,
+                                            datastore_version_name,
                                             config_file_location):
         print("Loading config parameters for datastore (%s) version (%s)"
-              % (datastore, datastore_version))
+              % (datastore, datastore_version_name))
         config_models.load_datastore_configuration_parameters(
-            datastore, datastore_version, config_file_location)
+            datastore, datastore_version_name, config_file_location)
+
+    def db_remove_datastore_config_parameters(self, datastore,
+                                              datastore_version_name):
+        print("Removing config parameters for datastore (%s) version (%s)"
+              % (datastore, datastore_version_name))
+        config_models.remove_datastore_configuration_parameters(
+            datastore, datastore_version_name)
 
     def datastore_version_flavor_add(self, datastore_name,
                                      datastore_version_name, flavor_ids):
@@ -223,13 +230,24 @@ def main():
             'datastore',
             help='Name of the datastore.')
         parser.add_argument(
-            'datastore_version',
+            'datastore_version_name',
             help='Name of the datastore version.')
         parser.add_argument(
             'config_file_location',
             help='Fully qualified file path to the configuration group '
             'parameter validation rules.')
 
+        parser = subparser.add_parser(
+            'db_remove_datastore_config_parameters',
+            description='Remove configuration group parameter validation '
+                        'rules for a datastore version from the database.')
+        parser.add_argument(
+            'datastore',
+            help='Name of the datastore.')
+        parser.add_argument(
+            'datastore_version_name',
+            help='Name of the datastore version.')
+
         parser = subparser.add_parser(
             'datastore_version_flavor_add', help='Adds flavor association to '
             'a given datastore and datastore version.')
