@@ -106,19 +106,39 @@ uuid = {
                "-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}$"
 }
 
+ip_address_v4 = {
+    "type": "string",
+    "minLength": 7,
+    "maxLength": 15,
+    "pattern": r"^(?:[0-9]{1,3}\.){3}[0-9]{1,3}$"
+}
+
 volume = {
     "type": "object",
     "required": ["size"],
     "properties": {
         "size": volume_size,
-        "type": non_empty_string
+        "type": {
+            "oneOf": [
+                non_empty_string,
+                {"type": "null"}
+            ]
+        }
     }
 }
 
 nics = {
     "type": "array",
+    "maxItems": 1,
     "items": {
         "type": "object",
+        "additionalProperties": False,
+        "properties": {
+            "net-id": uuid,
+            "network_id": uuid,
+            "subnet_id": uuid,
+            "ip_address": ip_address_v4
+        }
     }
 }
 
@@ -361,7 +381,7 @@ instance = {
         "properties": {
             "instance": {
                 "type": "object",
-                "required": ["name", "flavorRef"],
+                "required": ["name"],
                 "additionalProperties": True,
                 "properties": {
                     "name": non_empty_string,
@@ -378,6 +398,12 @@ instance = {
                             "backupRef": uuid
                         }
                     },
+                    "replica_of": uuid,
+                    "replica_count": {
+                        "type": "integer",
+                        "minimum": 1,
+                        "maximum": 3
+                    },
                     "availability_zone": non_empty_string,
                     "datastore": {
                         "type": "object",
@@ -390,7 +416,23 @@ instance = {
                     "nics": nics,
                     "modules": module_list,
                     "region_name": non_empty_string,
-                    "locality": non_empty_string
+                    "locality": non_empty_string,
+                    "access": {
+                        "type": "object",
+                        "properties": {
+                            "is_public": {"type": "boolean"},
+                            "allowed_cidrs": {
+                                "type": "array",
+                                "uniqueItems": True,
+                                "items": {
+                                    "type": "string",
+                                    "pattern": "^([0-9]{1,3}\\.){3}[0-9]{1,3}"
+                                               "(\\/([0-9]|[1-2][0-9]|3[0-2]))"
+                                               "?$"
+                                }
+                            }
+                        }
+                    }
                 }
             }
         }
@@ -414,6 +456,41 @@ instance = {
             }
         }
     },
+    "update": {
+        "name": "instance:update",
+        "type": "object",
+        "required": ["instance"],
+        "properties": {
+            "instance": {
+                "type": "object",
+                "required": [],
+                "additionalProperties": False,
+                "properties": {
+                    "name": non_empty_string,
+                    "replica_of": {},
+                    "configuration": configuration_id,
+                    "datastore_version": non_empty_string,
+                    "access": {
+                        "type": "object",
+                        "additionalProperties": False,
+                        "properties": {
+                            "is_public": {"type": "boolean"},
+                            "allowed_cidrs": {
+                                "type": "array",
+                                "uniqueItems": True,
+                                "items": {
+                                    "type": "string",
+                                    "pattern": "^([0-9]{1,3}\\.){3}[0-9]{1,3}"
+                                               "(\\/([0-9]|[1-2][0-9]|3[0-2]))"
+                                               "?$"
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    },
     "action": {
         "resize": {
             "volume": {
@@ -506,6 +583,21 @@ mgmt_instance = {
                     "type": "object"
                 }
             }
+        },
+        "rebuild": {
+            "type": "object",
+            "required": ["rebuild"],
+            "additionalProperties": True,
+            "properties": {
+                "rebuild": {
+                    "type": "object",
+                    "required": ["image_id"],
+                    "additionalProperties": False,
+                    "properties": {
+                        "image_id": uuid
+                    }
+                }
+            }
         }
     }
 }
@@ -565,13 +657,33 @@ backup = {
                     "instance": uuid,
                     "name": non_empty_string,
                     "parent_id": uuid,
-                    "incremental": boolean_string
+                    "incremental": boolean_string,
+                    "swift_container": non_empty_string
                 }
             }
         }
     }
 }
 
+backup_strategy = {
+    "create": {
+        "name": "backup_strategy:create",
+        "type": "object",
+        "required": ["backup_strategy"],
+        "properties": {
+            "backup_strategy": {
+                "type": "object",
+                "additionalProperties": False,
+                "required": ["swift_container"],
+                "properties": {
+                    "instance_id": uuid,
+                    "swift_container": non_empty_string
+                }
+            }
+        },
+    }
+}
+
 guest_log = {
     "action": {
         "name": "guest_log:action",
