@@ -54,8 +54,21 @@ def reset_management_networks():
     MGMT_NETWORKS = None
 
 
+def check_subnet_router(client, subnet_id):
+    """Check if the subnet is associated with a router."""
+    router_ports = client.list_ports(
+        device_owner="network:router_interface",
+        fixed_ips=f"subnet_id={subnet_id}")["ports"]
+    if not router_ports:
+        raise exception.TroveError(f"Subnet {subnet_id} is not "
+                                   f"associated with router.")
+
+
 def create_port(client, name, description, network_id, security_groups,
-                is_public=False):
+                is_public=False, subnet_id=None, ip=None, is_mgmt=False):
+    enable_access_check = (not is_mgmt and
+                           (CONF.network.enable_access_check or is_public))
+
     port_body = {
         "port": {
             "name": name,
@@ -64,21 +77,28 @@ def create_port(client, name, description, network_id, security_groups,
             "security_groups": security_groups
         }
     }
+
+    if subnet_id:
+        if enable_access_check:
+            check_subnet_router(client, subnet_id)
+
+        fixed_ips = {
+            "fixed_ips": [{"subnet_id": subnet_id}]
+        }
+        if ip:
+            fixed_ips['fixed_ips'][0].update({'ip_address': ip})
+        port_body['port'].update(fixed_ips)
+
     port = client.create_port(body=port_body)
     port_id = port['port']['id']
 
-    if is_public:
-        public_network_id = get_public_network(client)
-        if not public_network_id:
-            raise exception.PublicNetworkNotFound()
+    if not subnet_id and enable_access_check:
+        # Check if the subnet has been associated with a router.
+        subnet_id = port['port']['fixed_ips'][0]['subnet_id']
+        check_subnet_router(client, subnet_id)
 
-        fip_body = {
-            "floatingip": {
-                'floating_network_id': public_network_id,
-                'port_id': port_id,
-            }
-        }
-        client.create_floatingip(fip_body)
+    if is_public:
+        make_port_public(client, port_id)
 
     return port_id
 
@@ -98,6 +118,26 @@ def delete_port(client, id):
     client.delete_port(id)
 
 
+def make_port_public(client, port_id):
+    public_network_id = get_public_network(client)
+    if not public_network_id:
+        raise exception.PublicNetworkNotFound()
+
+    fip_body = {
+        "floatingip": {
+            'floating_network_id': public_network_id,
+            'port_id': port_id,
+        }
+    }
+
+    try:
+        client.create_floatingip(fip_body)
+    except Exception as e:
+        LOG.error(f"Failed to associate public IP with port {port_id}: "
+                  f"{str(e)}")
+        raise exception.TroveError('Failed to expose instance port to public.')
+
+
 def get_public_network(client):
     """Get public network ID.
 
@@ -116,6 +156,24 @@ def get_public_network(client):
     return ret['networks'][0].get('id')
 
 
+def ensure_port_access(client, port_id, is_public):
+    fips = client.list_floatingips(port_id=port_id)["floatingips"]
+
+    if is_public and not fips:
+        # Associate floating IP
+        LOG.debug(f"Associate public IP with port {port_id}")
+        make_port_public(client, port_id)
+        return
+
+    if not is_public and fips:
+        # Disassociate floating IP
+        for fip in fips:
+            LOG.debug(f"Disassociate public IP {fip['floating_ip_address']} "
+                      f"from port {port_id}")
+            client.delete_floatingip(fip["id"])
+        return
+
+
 def create_security_group(client, name, instance_id):
     body = {
         'security_group': {
@@ -150,12 +208,25 @@ def create_security_group_rule(client, sg_id, protocol, ports, remote_ips):
             client.create_security_group_rule(body)
 
 
-def get_subnet_cidrs(client, network_id):
+def clear_ingress_security_group_rules(client, sg_id):
+    rules = client.list_security_group_rules(
+        security_group_id=sg_id)['security_group_rules']
+
+    for rule in rules:
+        if rule['direction'] == 'ingress':
+            client.delete_security_group_rule(rule['id'])
+
+
+def get_subnet_cidrs(client, network_id=None, subnet_id=None):
     cidrs = []
 
-    subnets = client.list_subnets(network_id=network_id)['subnets']
-    for subnet in subnets:
-        cidrs.append(subnet.get('cidr'))
+    # Check subnet first.
+    if subnet_id:
+        cidrs.append(client.show_subnet(subnet_id)['subnet']['cidr'])
+    elif network_id:
+        subnets = client.list_subnets(network_id=network_id)['subnets']
+        for subnet in subnets:
+            cidrs.append(subnet.get('cidr'))
 
     return cidrs
 
