@@ -106,7 +106,8 @@ class MgmtInstanceController(InstanceController):
             'stop': self._action_stop,
             'reboot': self._action_reboot,
             'migrate': self._action_migrate,
-            'reset-task-status': self._action_reset_task_status
+            'reset-task-status': self._action_reset_task_status,
+            'rebuild': self._action_rebuild
         }
         selected_action = None
         for key in body:
@@ -131,7 +132,14 @@ class MgmtInstanceController(InstanceController):
 
     def _action_reboot(self, context, instance, req, body):
         LOG.debug("Rebooting instance %s.", instance.id)
-        instance.reboot()
+
+        context.notification = notification.DBaaSInstanceReboot(
+            context,
+            request=req
+        )
+        with StartNotification(context, instance_id=instance.id):
+            instance.reboot()
+
         return wsgi.Result(None, 202)
 
     def _action_migrate(self, context, instance, req, body):
@@ -154,6 +162,14 @@ class MgmtInstanceController(InstanceController):
 
         return wsgi.Result(None, 202)
 
+    def _action_rebuild(self, context, instance, req, body):
+        LOG.info("Rebuild instance %s.", instance.id)
+        req_body = body['rebuild']
+        image_id = req_body['image_id']
+
+        instance.rebuild(image_id)
+        return wsgi.Result(None, 202)
+
     @admin_context
     def root(self, req, tenant_id, id):
         """Return the date and time root was enabled on an instance,
