@@ -314,7 +314,7 @@ class API(object):
                 device_path='/dev/vdb', mount_point='/mnt/volume',
                 backup_info=None, config_contents=None, root_password=None,
                 overrides=None, cluster_config=None, snapshot=None,
-                modules=None):
+                modules=None, ds_version=None):
         """Make an asynchronous call to prepare the guest
            as a database container optionally includes a backup id for restores
         """
@@ -335,7 +335,8 @@ class API(object):
             device_path=device_path, mount_point=mount_point,
             backup_info=backup_info, config_contents=config_contents,
             root_password=root_password, overrides=overrides,
-            cluster_config=cluster_config, snapshot=snapshot, modules=modules)
+            cluster_config=cluster_config, snapshot=snapshot, modules=modules,
+            ds_version=ds_version)
 
     def _create_guest_queue(self):
         """Call to construct, start and immediately stop rpc server in order
@@ -379,6 +380,14 @@ class API(object):
                    self.agent_high_timeout, version=version,
                    upgrade_info=upgrade_info)
 
+    def upgrade(self, upgrade_info):
+        """Upgrade database service."""
+        LOG.debug("Sending the call to upgrade database service.")
+        version = self.API_BASE_VERSION
+
+        return self._cast("upgrade", version=version,
+                          upgrade_info=upgrade_info)
+
     def restart(self):
         """Restart the database server."""
         LOG.debug("Sending the call to restart the database process "
@@ -387,7 +396,7 @@ class API(object):
 
         self._call("restart", self.agent_high_timeout, version=version)
 
-    def start_db_with_conf_changes(self, config_contents):
+    def start_db_with_conf_changes(self, config_contents, ds_version):
         """Start the database server."""
         LOG.debug("Sending the call to start the database process on "
                   "the Guest with a timeout of %s.",
@@ -395,7 +404,8 @@ class API(object):
         version = self.API_BASE_VERSION
 
         self._call("start_db_with_conf_changes", self.agent_high_timeout,
-                   version=version, config_contents=config_contents)
+                   version=version, config_contents=config_contents,
+                   ds_version=ds_version)
 
     def reset_configuration(self, configuration):
         """Ignore running state of the database server; just change
@@ -409,25 +419,14 @@ class API(object):
         self._call("reset_configuration", self.agent_high_timeout,
                    version=version, configuration=configuration)
 
-    def stop_db(self, do_not_start_on_reboot=False):
+    def stop_db(self):
         """Stop the database server."""
         LOG.debug("Sending the call to stop the database process "
                   "on the Guest.")
         version = self.API_BASE_VERSION
 
-        self._call("stop_db", self.agent_high_timeout,
-                   version=version,
-                   do_not_start_on_reboot=do_not_start_on_reboot)
-
-    def upgrade(self, instance_version, location, metadata=None):
-        """Make an asynchronous call to self upgrade the guest agent."""
-        LOG.debug("Sending an upgrade call to nova-guest.")
-        version = self.API_BASE_VERSION
-
-        self._cast("upgrade", version=version,
-                   instance_version=instance_version,
-                   location=location,
-                   metadata=metadata)
+        self._call("stop_db", self.agent_low_timeout,
+                   version=version)
 
     def get_volume_info(self):
         """Make a synchronous call to get volume info for the container."""
@@ -475,7 +474,7 @@ class API(object):
                    self.agent_low_timeout, version=version,
                    device_path=device_path, mount_point=mount_point)
 
-    def resize_fs(self, device_path=None, mount_point=None):
+    def resize_fs(self, device_path=None, mount_point=None, online=False):
         """Resize the filesystem."""
         LOG.debug("Resize device %(device)s on instance %(id)s.", {
             'device': device_path, 'id': self.id})
@@ -483,7 +482,8 @@ class API(object):
 
         self._call("resize_fs",
                    self.agent_high_timeout, version=version,
-                   device_path=device_path, mount_point=mount_point)
+                   device_path=device_path, mount_point=mount_point,
+                   online=online)
 
     def update_overrides(self, overrides, remove=False):
         """Update the overrides."""
@@ -542,13 +542,14 @@ class API(object):
         return self._call("get_replica_context",
                           self.agent_high_timeout, version=version)
 
-    def attach_replica(self, replica_info, slave_config):
+    def attach_replica(self, replica_info, slave_config, restart=False):
         LOG.debug("Attaching replica %s.", replica_info)
         version = self.API_BASE_VERSION
 
         self._call("attach_replica",
                    self.agent_high_timeout, version=version,
-                   replica_info=replica_info, slave_config=slave_config)
+                   replica_info=replica_info, slave_config=slave_config,
+                   restart=restart)
 
     def make_read_only(self, read_only):
         LOG.debug("Executing make_read_only(%s)", read_only)
@@ -651,3 +652,18 @@ class API(object):
 
         return self._call("module_remove", self.agent_high_timeout,
                           version=version, module=module)
+
+    def rebuild(self, ds_version, config_contents=None, config_overrides=None):
+        """Make an asynchronous call to rebuild the database service."""
+        LOG.debug("Sending the call to rebuild database service in the guest.")
+        version = self.API_BASE_VERSION
+
+        # Taskmanager is a publisher, guestagent is a consumer. Usually
+        # consumer creates a queue, but in this case we have to make sure
+        # "prepare" doesn't get lost if for some reason guest was delayed and
+        # didn't create a queue on time.
+        self._create_guest_queue()
+
+        self._cast("rebuild", version=version,
+                   ds_version=ds_version, config_contents=config_contents,
+                   config_overrides=config_overrides)
