@@ -19,7 +19,12 @@ import re
 
 import six
 
+from trove.common import cfg
 from trove.common import pagination
+from trove.common import utils
+from trove.guestagent.common import operating_system
+
+CONF = cfg.CONF
 
 
 def update_dict(updates, target):
@@ -140,3 +145,39 @@ def serialize_list(li, limit=None, marker=None, include_marker=False):
     page, next_name = paginate_list(li, limit=limit, marker=marker,
                                     include_marker=include_marker)
     return [item.serialize() for item in page], next_name
+
+
+def get_filesystem_volume_stats(fs_path):
+    try:
+        stats = os.statvfs(fs_path)
+    except OSError:
+        raise RuntimeError("Filesystem not found (%s)" % fs_path)
+
+    total = stats.f_blocks * stats.f_bsize
+    free = stats.f_bfree * stats.f_bsize
+    # return the size in GB
+    used_gb = utils.to_gb(total - free)
+    total_gb = utils.to_gb(total)
+
+    output = {
+        'block_size': stats.f_bsize,
+        'total_blocks': stats.f_blocks,
+        'free_blocks': stats.f_bfree,
+        'total': total_gb,
+        'free': free,
+        'used': used_gb
+    }
+    return output
+
+
+def get_conf_dir():
+    """Get the config directory for the database related settings.
+
+    For now, the files inside the config dir are mainly for instance rebuild.
+    """
+    mount_point = CONF.get(CONF.datastore_manager).mount_point
+    conf_dir = os.path.join(mount_point, 'conf.d')
+    if not operating_system.exists(conf_dir, is_directory=True, as_root=True):
+        operating_system.ensure_directory(conf_dir, as_root=True)
+
+    return conf_dir
