@@ -32,7 +32,7 @@ from trove.common import pagination
 from trove.common import policy
 from trove.common import utils
 from trove.common import wsgi
-from trove.datastore import models as datastore_models
+from trove.datastore import models as ds_models
 from trove.extensions.mysql.common import populate_users
 from trove.extensions.mysql.common import populate_validated_databases
 from trove.instance import models, views
@@ -250,11 +250,11 @@ class InstanceController(wsgi.Controller):
         LOG.debug("req : '%s'\n\n", req)
 
         context = req.environ[wsgi.CONTEXT_KEY]
-        server = models.load_instance_with_info(models.DetailInstance,
-                                                context, id)
-        self.authorize_instance_action(context, 'show', server)
+        instance = models.load_instance_with_info(models.DetailInstance,
+                                                  context, id)
+        self.authorize_instance_action(context, 'show', instance)
         return wsgi.Result(
-            views.InstanceDetailView(server, req=req).data(), 200
+            views.InstanceDetailView(instance, req=req).data(), 200
         )
 
     def delete(self, req, tenant_id, id):
@@ -282,38 +282,140 @@ class InstanceController(wsgi.Controller):
             instance.delete()
         return wsgi.Result(None, 202)
 
-    def _check_network_overlap(self, context, user_network):
+    def _check_nic(self, context, nic):
+        """Check user provided nic.
+
+        :param context: User context.
+        :param nic: A dict may contain network_id(net-id), subnet_id or
+            ip_address.
+        """
         neutron_client = clients.create_neutron_client(context)
-        user_cidrs = neutron.get_subnet_cidrs(neutron_client, user_network)
+        network_id = nic.get('network_id', nic.get('net-id'))
+        subnet_id = nic.get('subnet_id')
+        ip_address = nic.get('ip_address')
+
+        if not network_id and not subnet_id:
+            raise exception.NetworkNotProvided(resource='network or subnet')
+
+        if not subnet_id and ip_address:
+            raise exception.NetworkNotProvided(resource='subnet')
+
+        if subnet_id:
+            actual_network = neutron_client.show_subnet(
+                subnet_id)['subnet']['network_id']
+            if network_id and actual_network != network_id:
+                raise exception.SubnetNotFound(subnet_id=subnet_id,
+                                               network_id=network_id)
+            network_id = actual_network
+
+        nic['network_id'] = network_id
+        nic.pop('net-id', None)
+
+        self._check_network_overlap(context, network_id, subnet_id)
+
+    def _check_network_overlap(self, context, user_network=None,
+                               user_subnet=None):
+        """Check if the network contains IP address belongs to reserved
+        network.
+
+        :param context: User context.
+        :param user_network: Network ID.
+        :param user_subnet: Subnet ID.
+        """
+        neutron_client = clients.create_neutron_client(context)
+        user_cidrs = neutron.get_subnet_cidrs(neutron_client, user_network,
+                                              user_subnet)
+
+        reserved_cidrs = CONF.reserved_network_cidrs
         mgmt_cidrs = neutron.get_mamangement_subnet_cidrs(neutron_client)
-        LOG.debug("Cidrs of the user network: %s, cidrs of the management "
-                  "network: %s", user_cidrs, mgmt_cidrs)
+        reserved_cidrs.extend(mgmt_cidrs)
+
+        LOG.debug("Cidrs of the user network: %s, cidrs of the reserved "
+                  "network: %s", user_cidrs, reserved_cidrs)
+
         for user_cidr in user_cidrs:
             user_net = ipaddress.ip_network(user_cidr)
-            for mgmt_cidr in mgmt_cidrs:
-                mgmt_net = ipaddress.ip_network(mgmt_cidr)
-                if user_net.overlaps(mgmt_net):
+            for reserved_cidr in reserved_cidrs:
+                res_net = ipaddress.ip_network(reserved_cidr)
+                if user_net.overlaps(res_net):
                     raise exception.NetworkConflict()
 
     def create(self, req, body, tenant_id):
-        # TODO(hub-cap): turn this into middleware
         LOG.info("Creating a database instance for tenant '%s'",
                  tenant_id)
         LOG.debug("req : '%s'\n\n", strutils.mask_password(req))
         LOG.debug("body : '%s'\n\n", strutils.mask_password(body))
         context = req.environ[wsgi.CONTEXT_KEY]
         policy.authorize_on_tenant(context, 'instance:create')
-        context.notification = notification.DBaaSInstanceCreate(context,
-                                                                request=req)
-        datastore_args = body['instance'].get('datastore', {})
-        datastore, datastore_version = (
-            datastore_models.get_datastore_version(**datastore_args))
-        image_id = datastore_version.image_id
+        context.notification = notification.DBaaSInstanceCreate(
+            context, request=req)
+
         name = body['instance']['name']
-        flavor_ref = body['instance']['flavorRef']
+        slave_of_id = body['instance'].get('replica_of')
+        replica_count = body['instance'].get('replica_count')
+        flavor_ref = body['instance'].get('flavorRef')
+        datastore_args = body['instance'].get('datastore', {})
+        volume_info = body['instance'].get('volume', {})
+        availability_zone = body['instance'].get('availability_zone')
+        nics = body['instance'].get('nics', [])
+        locality = body['instance'].get('locality')
+        region_name = body['instance'].get(
+            'region_name', CONF.service_credentials.region_name
+        )
+        access = body['instance'].get('access', None)
+
+        if slave_of_id:
+            if flavor_ref:
+                msg = 'Cannot specify flavor when creating replicas.'
+                raise exception.BadRequest(message=msg)
+            if datastore_args:
+                msg = 'Cannot specify datastore when creating replicas.'
+                raise exception.BadRequest(message=msg)
+            if volume_info:
+                msg = 'Cannot specify volume when creating replicas.'
+                raise exception.BadRequest(message=msg)
+            if locality:
+                msg = 'Cannot specify locality when creating replicas.'
+                raise exception.BadRequest(message=msg)
+            backup_model.verify_swift_auth_token(context)
+        else:
+            if replica_count and replica_count > 1:
+                msg = (f"Replica count only valid when creating replicas. "
+                       f"Cannot create {replica_count} instances.")
+                raise exception.BadRequest(message=msg)
+
         flavor_id = utils.get_id_from_href(flavor_ref)
 
-        configuration = self._configuration_parse(context, body)
+        if volume_info:
+            volume_size = int(volume_info.get('size'))
+            volume_type = volume_info.get('type')
+        else:
+            volume_size = None
+            volume_type = None
+
+        if slave_of_id:
+            try:
+                replica_source = models.DBInstance.find_by(
+                    context, id=slave_of_id, deleted=False)
+                flavor_id = replica_source.flavor_id
+            except exception.ModelNotFoundError:
+                LOG.error(f"Cannot create a replica of {slave_of_id} as that "
+                          f"instance could not be found.")
+                raise exception.NotFound(uuid=slave_of_id)
+            if replica_source.slave_of_id:
+                raise exception.Forbidden(
+                    f"Cannot create a replica of a replica {slave_of_id}")
+
+            datastore_version = ds_models.DatastoreVersion.load_by_uuid(
+                replica_source.datastore_version_id)
+            datastore = ds_models.Datastore.load(
+                datastore_version.datastore_id)
+        else:
+            datastore, datastore_version = ds_models.get_datastore_version(
+                **datastore_args)
+
+        image_id = datastore_version.image_id
+
         databases = populate_validated_databases(
             body['instance'].get('databases', []))
         database_names = [database.get('_name', '') for database in databases]
@@ -323,7 +425,10 @@ class InstanceController(wsgi.Controller):
                                    database_names)
         except ValueError as ve:
             raise exception.BadRequest(message=ve)
+        if slave_of_id and (databases or users):
+            raise exception.ReplicaCreateWithUsersDatabasesError()
 
+        configuration = self._configuration_parse(context, body)
         modules = body['instance'].get('modules')
 
         # The following operations have their own API calls.
@@ -343,31 +448,22 @@ class InstanceController(wsgi.Controller):
             policy.authorize_on_tenant(
                 context, 'instance:extension:database:create')
 
-        if 'volume' in body['instance']:
-            volume_info = body['instance']['volume']
-            volume_size = int(volume_info['size'])
-            volume_type = volume_info.get('type')
-        else:
-            volume_size = None
-            volume_type = None
-
         if 'restorePoint' in body['instance']:
             backupRef = body['instance']['restorePoint']['backupRef']
             backup_id = utils.get_id_from_href(backupRef)
         else:
             backup_id = None
 
-        availability_zone = body['instance'].get('availability_zone')
-
-        nics = body['instance'].get('nics', [])
+        # Only 1 nic is allowed as defined in API jsonschema.
+        # Use list just for backward compatibility.
         if len(nics) > 0:
-            self._check_network_overlap(context, nics[0].get('net-id'))
+            nic = nics[0]
+            LOG.info('Checking user provided instance network %s', nic)
+            if slave_of_id and nic.get('ip_address'):
+                msg = "Cannot specify IP address when creating replicas."
+                raise exception.BadRequest(message=msg)
+            self._check_nic(context, nic)
 
-        slave_of_id = body['instance'].get('replica_of',
-                                           # also check for older name
-                                           body['instance'].get('slave_of'))
-        replica_count = body['instance'].get('replica_count')
-        locality = body['instance'].get('locality')
         if locality:
             locality_domain = ['affinity', 'anti-affinity']
             locality_domain_msg = ("Invalid locality '%s'. "
@@ -376,16 +472,6 @@ class InstanceController(wsgi.Controller):
                                     "', '".join(locality_domain)))
             if locality not in locality_domain:
                 raise exception.BadRequest(message=locality_domain_msg)
-            if slave_of_id:
-                dupe_locality_msg = (
-                    'Cannot specify locality when adding replicas to existing '
-                    'master.')
-                raise exception.BadRequest(message=dupe_locality_msg)
-
-        region_name = body['instance'].get(
-            'region_name', CONF.service_credentials.region_name
-        )
-        access = body['instance'].get('access', None)
 
         instance = models.Instance.create(context, name, flavor_id,
                                           image_id, databases, users,
@@ -409,15 +495,15 @@ class InstanceController(wsgi.Controller):
             if configuration_ref:
                 configuration_id = utils.get_id_from_href(configuration_ref)
                 return configuration_id
+            return ""
 
     def _modify_instance(self, context, req, instance, **kwargs):
         if 'detach_replica' in kwargs and kwargs['detach_replica']:
-            LOG.debug("Detaching replica from source.")
             context.notification = notification.DBaaSInstanceDetach(
                 context, request=req)
             with StartNotification(context, instance_id=instance.id):
                 instance.detach_replica()
-        if 'configuration_id' in kwargs:
+        elif 'configuration_id' in kwargs:
             if kwargs['configuration_id']:
                 context.notification = (
                     notification.DBaaSInstanceAttachConfiguration(context,
@@ -432,19 +518,24 @@ class InstanceController(wsgi.Controller):
                                                                   request=req))
                 with StartNotification(context, instance_id=instance.id):
                     instance.detach_configuration()
-        if 'datastore_version' in kwargs:
-            datastore_version = datastore_models.DatastoreVersion.load(
+        elif 'datastore_version' in kwargs:
+            datastore_version = ds_models.DatastoreVersion.load(
                 instance.datastore, kwargs['datastore_version'])
             context.notification = (
                 notification.DBaaSInstanceUpgrade(context, request=req))
             with StartNotification(context, instance_id=instance.id,
                                    datastore_version_id=datastore_version.id):
                 instance.upgrade(datastore_version)
-        if kwargs:
-            instance.update_db(**kwargs)
+        elif 'access' in kwargs:
+            instance.update_access(kwargs['access'])
 
     def update(self, req, id, body, tenant_id):
-        """Updates the instance to attach/detach configuration."""
+        """Updates the instance.
+
+        - attach/detach configuration.
+        - detach from primary.
+        - access information.
+        """
         LOG.info("Updating database instance '%(instance_id)s' for tenant "
                  "'%(tenant_id)s'",
                  {'instance_id': id, 'tenant_id': tenant_id})
@@ -452,41 +543,40 @@ class InstanceController(wsgi.Controller):
         LOG.debug("body: %s", body)
         context = req.environ[wsgi.CONTEXT_KEY]
 
+        name = body['instance'].get('name')
+        if ((name and len(body['instance'].keys()) > 2) or
+            (not name and len(body['instance'].keys()) >= 2)):
+            raise exception.BadRequest("Only one attribute (except 'name') is "
+                                       "allowed to update.")
+
         instance = models.Instance.load(context, id)
         self.authorize_instance_action(context, 'update', instance)
 
-        # Make sure args contains a 'configuration_id' argument,
         args = {}
-        args['configuration_id'] = self._configuration_parse(context, body)
+        if name:
+            instance.update_db(name=name)
+
+        detach_replica = ('replica_of' in body['instance'] or
+                          'slave_of' in body['instance'])
+        if detach_replica:
+            args['detach_replica'] = detach_replica
+
+        configuration_id = self._configuration_parse(context, body)
+        if configuration_id is not None:
+            args['configuration_id'] = configuration_id
+
+        if 'access' in body['instance']:
+            args['access'] = body['instance']['access']
+
         self._modify_instance(context, req, instance, **args)
         return wsgi.Result(None, 202)
 
     def edit(self, req, id, body, tenant_id):
+        """Updates the instance to set or unset one or more attributes.
+
+        Deprecated. Use update method instead.
         """
-        Updates the instance to set or unset one or more attributes.
-        """
-        LOG.info("Editing instance for tenant id %s.", tenant_id)
-        LOG.debug("req: %s", strutils.mask_password(req))
-        LOG.debug("body: %s", strutils.mask_password(body))
-        context = req.environ[wsgi.CONTEXT_KEY]
-
-        instance = models.Instance.load(context, id)
-        self.authorize_instance_action(context, 'edit', instance)
-
-        args = {}
-        args['detach_replica'] = ('replica_of' in body['instance'] or
-                                  'slave_of' in body['instance'])
-
-        if 'name' in body['instance']:
-            args['name'] = body['instance']['name']
-        if 'configuration' in body['instance']:
-            args['configuration_id'] = self._configuration_parse(context, body)
-        if 'datastore_version' in body['instance']:
-            args['datastore_version'] = body['instance'].get(
-                'datastore_version')
-
-        self._modify_instance(context, req, instance, **args)
-        return wsgi.Result(None, 202)
+        self.update(req, id, body, tenant_id)
 
     def configuration(self, req, tenant_id, id):
         """
