@@ -158,17 +158,23 @@ class ActionTestBase(object):
             self.instance_id,
             ip_address=self.instance_mgmt_address
         )
-        cmd = "sudo ps acux | grep mysqld " \
-              "| grep -v mysqld_safe | awk '{print $2}'"
+        container_exist_cmd = 'sudo docker ps -q'
+        pid_cmd = "sudo docker inspect database -f '{{.State.Pid}}'"
 
         try:
-            stdout = server.execute(cmd)
+            server.execute(container_exist_cmd)
+        except Exception as err:
+            asserts.fail("Failed to execute command: %s, error: %s" %
+                         (container_exist_cmd, str(err)))
+
+        try:
+            stdout = server.execute(pid_cmd)
             return int(stdout)
         except ValueError:
             return None
-        except Exception as e:
+        except Exception as err:
             asserts.fail("Failed to execute command: %s, error: %s" %
-                         (cmd, str(e)))
+                         (pid_cmd, str(err)))
 
     def log_current_users(self):
         users = self.dbaas.users.list(self.instance_id)
@@ -261,6 +267,15 @@ class RebootTestBase(ActionTestBase):
 
         poll_until(is_finished_rebooting, time_out=TIME_OUT_TIME)
 
+    def wait_for_status(self, status, timeout=60, sleep_time=5):
+        def is_status():
+            instance = self.instance
+            if instance.status in status:
+                return True
+            return False
+
+        poll_until(is_status, time_out=timeout, sleep_time=sleep_time)
+
 
 @test(groups=[tests.DBAAS_API_INSTANCE_ACTIONS],
       depends_on_groups=[tests.DBAAS_API_DATABASES],
@@ -306,9 +321,11 @@ class StopTests(RebootTestBase):
 
     @test(depends_on=[test_ensure_mysql_is_running])
     def test_stop_mysql(self):
-        """Stops MySQL."""
+        """Stops MySQL by admin."""
         instance_info.dbaas_admin.management.stop(self.instance_id)
-        self.wait_for_failure_status()
+
+        # The instance status will only be updated by guest agent.
+        self.wait_for_status(['SHUTDOWN'], timeout=90, sleep_time=10)
 
     @test(depends_on=[test_stop_mysql])
     def test_volume_info_while_mysql_is_down(self):
@@ -367,11 +384,6 @@ class ResizeInstanceTest(ActionTestBase):
     def flavor_id(self):
         return instance_info.dbaas_flavor_href
 
-    def get_flavor_href(self, flavor_id=2):
-        res = instance_info.dbaas.find_flavor_and_self_href(flavor_id)
-        _, dbaas_flavor_href = res
-        return dbaas_flavor_href
-
     def wait_for_resize(self):
         def is_finished_resizing():
             instance = self.instance
@@ -399,7 +411,12 @@ class ResizeInstanceTest(ActionTestBase):
     def test_instance_resize_to_ephemeral_in_volume_support_should_fail(self):
         flavor_name = CONFIG.values.get('instance_bigger_eph_flavor_name',
                                         'eph.rd-smaller')
-        flavors = self.dbaas.find_flavors_by_name(flavor_name)
+        flavor_id = None
+        for item in instance_info.flavors:
+            if item.name == flavor_name:
+                flavor_id = item.id
+
+        asserts.assert_is_not_none(flavor_id)
 
         def is_active():
             return self.instance.status in CONFIG.running_status
@@ -409,36 +426,42 @@ class ResizeInstanceTest(ActionTestBase):
 
         asserts.assert_raises(HTTPNotImplemented,
                               self.dbaas.instances.resize_instance,
-                              self.instance_id, flavors[0].id)
+                              self.instance_id, flavor_id)
 
     @test(enabled=EPHEMERAL_SUPPORT)
     def test_instance_resize_to_non_ephemeral_flavor_should_fail(self):
         flavor_name = CONFIG.values.get('instance_bigger_flavor_name',
                                         'm1-small')
-        flavors = self.dbaas.find_flavors_by_name(flavor_name)
+        flavor_id = None
+        for item in instance_info.flavors:
+            if item.name == flavor_name:
+                flavor_id = item.id
+
+        asserts.assert_is_not_none(flavor_id)
         asserts.assert_raises(BadRequest, self.dbaas.instances.resize_instance,
-                              self.instance_id, flavors[0].id)
+                              self.instance_id, flavor_id)
 
     def obtain_flavor_ids(self):
         old_id = self.instance.flavor['id']
         self.expected_old_flavor_id = old_id
-        res = instance_info.dbaas.find_flavor_and_self_href(old_id)
-        self.expected_dbaas_flavor, _ = res
         if EPHEMERAL_SUPPORT:
             flavor_name = CONFIG.values.get('instance_bigger_eph_flavor_name',
                                             'eph.rd-smaller')
         else:
             flavor_name = CONFIG.values.get('instance_bigger_flavor_name',
                                             'm1.small')
-        flavors = self.dbaas.find_flavors_by_name(flavor_name)
-        asserts.assert_equal(len(flavors), 1,
-                             "Number of flavors with name '%s' "
-                             "found was '%d'." % (flavor_name,
-                                                  len(flavors)))
-        flavor = flavors[0]
+
+        new_flavor = None
+        for item in instance_info.flavors:
+            if item.name == flavor_name:
+                new_flavor = item
+                break
+
+        asserts.assert_is_not_none(new_flavor)
+
         self.old_dbaas_flavor = instance_info.dbaas_flavor
-        instance_info.dbaas_flavor = flavor
-        self.expected_new_flavor_id = flavor.id
+        instance_info.dbaas_flavor = new_flavor
+        self.expected_new_flavor_id = new_flavor.id
 
     @test(depends_on=[test_instance_resize_same_size_should_fail])
     def test_status_changed_to_resize(self):
@@ -447,14 +470,14 @@ class ResizeInstanceTest(ActionTestBase):
         self.obtain_flavor_ids()
         self.dbaas.instances.resize_instance(
             self.instance_id,
-            self.get_flavor_href(flavor_id=self.expected_new_flavor_id))
+            self.expected_new_flavor_id)
         asserts.assert_equal(202, self.dbaas.last_http_code)
 
         # (WARNING) IF THE RESIZE IS WAY TOO FAST THIS WILL FAIL
         assert_unprocessable(
             self.dbaas.instances.resize_instance,
             self.instance_id,
-            self.get_flavor_href(flavor_id=self.expected_new_flavor_id))
+            self.expected_new_flavor_id)
 
     @test(depends_on=[test_status_changed_to_resize])
     @time_out(TIME_OUT_TIME)
@@ -463,8 +486,7 @@ class ResizeInstanceTest(ActionTestBase):
         self.wait_for_resize()
 
     @test(depends_on=[test_instance_returns_to_active_after_resize,
-                      test_status_changed_to_resize],
-          groups=["dbaas.usage"])
+                      test_status_changed_to_resize])
     def test_resize_instance_usage_event_sent(self):
         expected = self._build_expected_msg()
         expected['old_instance_size'] = self.old_dbaas_flavor.ram
@@ -493,9 +515,8 @@ class ResizeInstanceTest(ActionTestBase):
 
     @test(depends_on=[test_make_sure_mysql_is_running_after_resize])
     def test_instance_has_new_flavor_after_resize(self):
-        actual = self.get_flavor_href(self.instance.flavor['id'])
-        expected = self.get_flavor_href(flavor_id=self.expected_new_flavor_id)
-        asserts.assert_equal(actual, expected)
+        actual = self.instance.flavor['id']
+        asserts.assert_equal(actual, self.expected_new_flavor_id)
 
 
 @test(depends_on_classes=[ResizeInstanceTest],
@@ -520,28 +541,31 @@ class ResizeInstanceVolumeTest(ActionTestBase):
     @test
     @time_out(60)
     def test_volume_resize(self):
+        """test_volume_resize"""
         instance_info.dbaas.instances.resize_volume(instance_info.id,
                                                     self.new_volume_size)
 
     @test(depends_on=[test_volume_resize])
-    @time_out(300)
     def test_volume_resize_success(self):
+        """test_volume_resize_success"""
 
         def check_resize_status():
             instance = instance_info.dbaas.instances.get(instance_info.id)
             if instance.status in CONFIG.running_status:
                 return True
-            elif instance.status == "RESIZE":
+            elif instance.status in ["RESIZE", "SHUTDOWN"]:
                 return False
             else:
                 asserts.fail("Status should not be %s" % instance.status)
 
-        poll_until(check_resize_status, sleep_time=2, time_out=300)
+        poll_until(check_resize_status, sleep_time=5, time_out=300,
+                   initial_delay=5)
         instance = instance_info.dbaas.instances.get(instance_info.id)
         asserts.assert_equal(instance.volume['size'], self.new_volume_size)
 
     @test(depends_on=[test_volume_resize_success])
     def test_volume_filesystem_resize_success(self):
+        """test_volume_filesystem_resize_success"""
         # The get_volume_filesystem_size is a mgmt call through the guestagent
         # and the volume resize occurs through the fake nova-volume.
         # Currently the guestagent fakes don't have access to the nova fakes so
@@ -555,8 +579,9 @@ class ResizeInstanceVolumeTest(ActionTestBase):
         # cinder volume but it should round to it. (e.g. round(1.9) == 2)
         asserts.assert_equal(round(new_volume_fs_size), self.new_volume_size)
 
-    @test(depends_on=[test_volume_resize_success], groups=["dbaas.usage"])
+    @test(depends_on=[test_volume_resize_success])
     def test_resize_volume_usage_event_sent(self):
+        """test_resize_volume_usage_event_sent"""
         expected = self._build_expected_msg()
         expected['volume_size'] = self.new_volume_size
         expected['old_volume_size'] = self.old_volume_size
@@ -564,9 +589,9 @@ class ResizeInstanceVolumeTest(ActionTestBase):
                                              'trove.instance.modify_volume',
                                              **expected)
 
-    @test
-    @time_out(300)
+    @test(depends_on=[test_volume_resize_success])
     def test_volume_resize_success_databases(self):
+        """test_volume_resize_success_databases"""
         databases = instance_info.dbaas.databases.list(instance_info.id)
         db_list = []
         for database in databases:
