@@ -14,7 +14,7 @@
 #    under the License.
 
 from hashlib import md5
-from mock import MagicMock, patch
+from unittest.mock import MagicMock, patch
 import json
 import os
 import socket
@@ -37,7 +37,7 @@ class FakeSwiftClient(object):
         pass
 
     @classmethod
-    def Connection(self, *args, **kargs):
+    def Connection(cls, *args, **kargs):
         LOG.debug("fake FakeSwiftClient Connection")
         return FakeSwiftConnection()
 
@@ -108,10 +108,7 @@ class FakeSwiftConnection(object):
                 object_checksum = md5(self.container_objects[object_name])
                 # The manifest file etag for a HEAD or GET is the checksum of
                 # the concatenated checksums.
-                if six.PY3:
-                    checksum.update(object_checksum.hexdigest().encode())
-                else:
-                    checksum.update(object_checksum.hexdigest())
+                checksum.update(object_checksum.hexdigest().encode())
             # this is included to test bad swift segment etags
             if name.startswith("bad_manifest_etag_"):
                 return {'etag': '"this_is_an_intentional_bad_manifest_etag"'}
