@@ -18,7 +18,7 @@ Tests dealing with HTTP rate-limiting.
 """
 
 
-from mock import Mock, MagicMock, patch
+from unittest.mock import Mock, MagicMock, patch
 from oslo_serialization import jsonutils
 import six
 from six.moves import http_client
@@ -307,36 +307,38 @@ class ParseLimitsTest(BaseLimitTestSuite):
     def test_multiple_rules(self):
         # Test that parse_limits() handles multiple rules correctly.
         try:
-            l = limits.Limiter.parse_limits('(get, *, .*, 20, minute);'
-                                            '(PUT, /foo*, /foo.*, 10, hour);'
-                                            '(POST, /bar*, /bar.*, 5, second);'
-                                            '(Say, /derp*, /derp.*, 1, day)')
+            result = limits.Limiter.parse_limits(
+                '(get, *, .*, 20, minute);'
+                '(PUT, /foo*, /foo.*, 10, hour);'
+                '(POST, /bar*, /bar.*, 5, second);'
+                '(Say, /derp*, /derp.*, 1, day)'
+            )
         except ValueError as e:
             assert False, str(e)
 
         # Make sure the number of returned limits are correct
-        self.assertEqual(4, len(l))
+        self.assertEqual(4, len(result))
 
         # Check all the verbs...
         expected = ['GET', 'PUT', 'POST', 'SAY']
-        self.assertEqual(expected, [t.verb for t in l])
+        self.assertEqual(expected, [t.verb for t in result])
 
         # ...the URIs...
         expected = ['*', '/foo*', '/bar*', '/derp*']
-        self.assertEqual(expected, [t.uri for t in l])
+        self.assertEqual(expected, [t.uri for t in result])
 
         # ...the regexes...
         expected = ['.*', '/foo.*', '/bar.*', '/derp.*']
-        self.assertEqual(expected, [t.regex for t in l])
+        self.assertEqual(expected, [t.regex for t in result])
 
         # ...the values...
         expected = [20, 10, 5, 1]
-        self.assertEqual(expected, [t.value for t in l])
+        self.assertEqual(expected, [t.value for t in result])
 
         # ...and the units...
         expected = [limits.PER_MINUTE, limits.PER_HOUR,
                     limits.PER_SECOND, limits.PER_DAY]
-        self.assertEqual(expected, [t.unit for t in l])
+        self.assertEqual(expected, [t.unit for t in result])
 
 
 class LimiterTest(BaseLimitTestSuite):
@@ -598,8 +600,7 @@ class FakeHttplibConnection(object):
 
         resp = str(req.get_response(self.app))
         resp = "HTTP/1.0 %s" % resp
-        if six.PY3:
-            resp = resp.encode("utf-8")
+        resp = resp.encode("utf-8")
         sock = FakeHttplibSocket(resp)
         self.http_response = http_client.HTTPResponse(sock)
         self.http_response.begin()
