@@ -14,8 +14,8 @@
 #    under the License.
 #
 
-from mock import Mock
-from mock import patch
+from unittest.mock import Mock
+from unittest.mock import patch
 
 from testtools import ExpectedException
 from trove.common import exception
