@@ -13,32 +13,37 @@
 #    under the License.
 import os
 from tempfile import NamedTemporaryFile
+from unittest import mock
+from unittest.mock import call
+from unittest.mock import MagicMock
+from unittest.mock import Mock
+from unittest.mock import patch
+from unittest.mock import PropertyMock
 
 from cinderclient import exceptions as cinder_exceptions
-import cinderclient.v2.client as cinderclient
 from cinderclient.v2 import volumes as cinderclient_volumes
-import mock
-from mock import Mock, MagicMock, patch, PropertyMock, call
+import cinderclient.v2.client as cinderclient
 import neutronclient.v2_0.client as neutronclient
 from novaclient import exceptions as nova_exceptions
 import novaclient.v2.flavors
 import novaclient.v2.servers
 from oslo_config import cfg
 from swiftclient.client import ClientException
-from testtools.matchers import Equals, Is
+from testtools.matchers import Equals
+from testtools.matchers import Is
 
-import trove.backup.models
+from trove import rpc
 from trove.backup import models as backup_models
 from trove.backup import state
+import trove.backup.models
+from trove.common import timeutils
+from trove.common import utils
 import trove.common.context
+from trove.common import exception
 from trove.common.exception import GuestError
 from trove.common.exception import PollTimeOut
 from trove.common.exception import TroveError
-from trove.common.instance import ServiceStatuses
-from trove.common.notification import TroveInstanceModifyVolume
 import trove.common.template as template
-from trove.common import timeutils
-from trove.common import utils
 from trove.datastore import models as datastore_models
 import trove.db.models
 from trove.extensions.common import models as common_models
@@ -48,8 +53,8 @@ from trove.instance.models import BaseInstance
 from trove.instance.models import DBInstance
 from trove.instance.models import InstanceServiceStatus
 from trove.instance.models import InstanceStatus
+from trove.instance.service_status import ServiceStatuses
 from trove.instance.tasks import InstanceTasks
-from trove import rpc
 from trove.taskmanager import models as taskmanager_models
 from trove.tests.unittests import trove_testtools
 from trove.tests.unittests.util import util
@@ -244,7 +249,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
         cfg.CONF.set_override('injected_config_location', '/etc/trove/conf.d')
 
         # execute
-        files = self.freshinstancetasks.get_injected_files("test")
+        files = self.freshinstancetasks.get_injected_files("test", 'test')
         # verify
         self.assertTrue(
             '/etc/trove/conf.d/guest_info.conf' in files)
@@ -261,7 +266,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
         cfg.CONF.set_override('injected_config_location', '/etc')
 
         # execute
-        files = self.freshinstancetasks.get_injected_files("test")
+        files = self.freshinstancetasks.get_injected_files("test", 'test')
         # verify
         self.assertTrue(
             '/etc/guest_info' in files)
@@ -296,7 +301,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
                   new_callable=PropertyMock,
                   return_value='fake-hostname')
     def test_servers_create_block_device_mapping_v2(self, mock_hostname):
-        self.freshinstancetasks._prepare_userdata = Mock(return_value=None)
+        self.freshinstancetasks.prepare_userdata = Mock(return_value=None)
         mock_nova_client = self.freshinstancetasks.nova_client = Mock()
         mock_servers_create = mock_nova_client.servers.create
         self.freshinstancetasks._create_server('fake-flavor', 'fake-image',
@@ -367,7 +372,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
 
         mock_create_secgroup.assert_called_with('mysql', [])
         mock_create_port.assert_called_once_with(
-            'fake-net-id',
+            {'net-id': 'fake-net-id'},
             ['fake_security_group_id'],
             is_mgmt=False,
             is_public=False
@@ -378,7 +383,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
         )
         mock_guest_prepare.assert_called_with(
             768, mock_build_volume_info(), 'mysql-server', None, None, None,
-            config_content, None, overrides, None, None, None
+            config_content, None, overrides, None, None, None, ds_version=None
         )
         mock_create_server.assert_called_with(
             8, 'mysql-image-id', 'mysql',
@@ -412,12 +417,21 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
         }
         mock_client.create_port.side_effect = [
             {'port': {'id': 'fake-mgmt-port-id'}},
-            {'port': {'id': 'fake-user-port-id'}}
+            {
+                'port': {
+                    'id': 'fake-user-port-id',
+                    'fixed_ips': [{'subnet_id': 'fake-subnet-id'}]
+                }
+            }
         ]
         mock_client.list_networks.return_value = {
             'networks': [{'id': 'fake-public-net-id'}]
         }
+        mock_client.list_ports.return_value = {
+            'ports': [{'id': 'fake-port-id'}]
+        }
         mock_neutron_client.return_value = mock_client
+
         mock_flavor = {'id': 8, 'ram': 768, 'name': 'bigger_flavor'}
         config_content = {'config_contents': 'some junk'}
         mock_single_instance_template.return_value.config_contents = (
@@ -440,7 +454,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
         )
         mock_guest_prepare.assert_called_with(
             768, mock_build_volume_info(), 'mysql-server', None, None, None,
-            config_content, None, mock.ANY, None, None, None)
+            config_content, None, mock.ANY, None, None, None, ds_version=None)
         mock_create_server.assert_called_with(
             8, 'mysql-image-id', 'mysql',
             mock_build_volume_info()['block_device'], None,
@@ -622,11 +636,10 @@ class ResizeVolumeTest(trove_testtools.TestCase):
         self.instance.volume_client.volumes.extend.side_effect = None
         self.instance.reset_mock()
 
-    @patch('trove.taskmanager.models.LOG')
-    def test_resize_volume_verify_extend_no_volume(self, mock_logging):
+    def test_resize_volume_verify_extend_no_volume(self):
         self.instance.volume_client.volumes.get = Mock(
             return_value=None)
-        self.assertRaises(cinder_exceptions.ClientException,
+        self.assertRaises(exception.TroveError,
                           self.action._verify_extend)
         self.instance.reset_mock()
 
@@ -638,29 +651,20 @@ class ResizeVolumeTest(trove_testtools.TestCase):
         utils.poll_until.side_effect = None
         self.instance.reset_mock()
 
-    @patch.object(TroveInstanceModifyVolume, 'notify')
     def test_resize_volume_active_server_succeeds(self, *args):
         server = Mock(status=InstanceStatus.ACTIVE)
         self.instance.attach_mock(server, 'server')
+
         self.action.execute()
-        self.assertEqual(1, self.instance.guest.stop_db.call_count)
-        self.assertEqual(1, self.instance.guest.unmount_volume.call_count)
-        detach_count = (
-            self.instance.nova_client.volumes.delete_server_volume.call_count)
-        self.assertEqual(1, detach_count)
+
         extend_count = self.instance.volume_client.volumes.extend.call_count
         self.assertEqual(1, extend_count)
-        attach_count = (
-            self.instance.nova_client.volumes.create_server_volume.call_count)
-        self.assertEqual(1, attach_count)
-        self.assertEqual(1, self.instance.guest.resize_fs.call_count)
-        self.assertEqual(1, self.instance.guest.mount_volume.call_count)
-        self.assertEqual(1, self.instance.restart.call_count)
         self.instance.reset_mock()
 
     def test_resize_volume_server_error_fails(self):
         server = Mock(status=InstanceStatus.ERROR)
         self.instance.attach_mock(server, 'server')
+
         self.assertRaises(TroveError, self.action.execute)
         self.instance.reset_mock()
 
@@ -703,7 +707,7 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
     def setUp(self):
         super(BuiltInstanceTasksTest, self).setUp()
         self.new_flavor = {'id': 8, 'ram': 768, 'name': 'bigger_flavor'}
-        stub_nova_server = MagicMock()
+        stub_nova_server = MagicMock(id='fake_id')
         self.rpc_patches = patch.multiple(
             rpc, get_notifier=MagicMock(), get_client=MagicMock())
         self.rpc_mocks = self.rpc_patches.start()
@@ -754,6 +758,7 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
         self.stub_running_server.flavor = {'id': 6, 'ram': 512}
         self.stub_verifying_server = MagicMock(
             spec=novaclient.v2.servers.Server)
+        self.stub_verifying_server.id = 'fake_id'
         self.stub_verifying_server.status = 'VERIFY_RESIZE'
         self.stub_verifying_server.flavor = {'id': 8, 'ram': 768}
         self.stub_server_mgr.get = MagicMock(
@@ -822,11 +827,10 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
                                          self.new_flavor)
         # verify
         self.assertIsNot(self.instance_task.server, orig_server)
-        self.instance_task._guest.stop_db.assert_any_call(
-            do_not_start_on_reboot=True)
+        self.assertEqual(1, self.instance_task._guest.stop_db.call_count)
         orig_server.resize.assert_any_call(self.new_flavor['id'])
         self.assertThat(self.db_instance.task_status, Is(InstanceTasks.NONE))
-        self.assertEqual(1, self.stub_server_mgr.get.call_count)
+        self.assertEqual(2, self.stub_server_mgr.get.call_count)
         self.assertThat(self.db_instance.flavor_id, Is(self.new_flavor['id']))
 
     @patch('trove.taskmanager.models.LOG')
@@ -842,8 +846,7 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
             self.assertTrue(self.stub_server_mgr.get.called)
             self.assertIs(self.instance_task.server,
                           self.stub_verifying_server)
-            self.instance_task._guest.stop_db.assert_any_call(
-                do_not_start_on_reboot=True)
+            self.assertEqual(1, self.instance_task._guest.stop_db.call_count)
             orig_server.resize.assert_any_call(self.new_flavor['id'])
             self.assertThat(self.db_instance.task_status,
                             Is(InstanceTasks.NONE))
@@ -852,11 +855,12 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
     @patch.object(utils, 'poll_until')
     def test_reboot(self, mock_poll):
         self.instance_task.server.reboot = Mock()
-        self.instance_task.set_datastore_status_to_paused = Mock()
+
         self.instance_task.reboot()
+
         self.instance_task._guest.stop_db.assert_any_call()
         self.instance_task.server.reboot.assert_any_call()
-        self.instance_task.set_datastore_status_to_paused.assert_any_call()
+        self.instance_task._guest.restart.assert_any_call()
 
     @patch.object(BaseInstance, 'update_db')
     def test_detach_replica(self, mock_update_db):
@@ -916,7 +920,7 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
                           return_value=replica_config):
             self.instance_task.attach_replica(master)
         self.instance_task._guest.attach_replica.assert_called_with(
-            replica_context, config_content)
+            replica_context, config_content, restart=False)
         mock_update_db.assert_called_with(slave_of_id=master.id)
 
     @patch('trove.taskmanager.models.LOG')
@@ -926,34 +930,6 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
             self.assertRaises(GuestError, self.instance_task.attach_replica,
                               Mock())
 
-    def test_get_floating_ips(self):
-        floating_ips = self.instance_task._get_floating_ips()
-        self.assertEqual({'192.168.10.1': 'fake-floatingip-id'},
-                         floating_ips)
-
-    @patch.object(BaseInstance, 'get_visible_ip_addresses',
-                  return_value=['192.168.10.1'])
-    def test_detach_public_ips(self, mock_address):
-        removed_ips = self.instance_task.detach_public_ips()
-        self.assertEqual(['fake-floatingip-id'], removed_ips)
-        mock_update_floatingip = (self.instance_task.neutron_client
-                                  .update_floatingip)
-        mock_update_floatingip.assert_called_once_with(
-            removed_ips[0], {'floatingip': {'port_id': None}})
-
-    def test_attach_public_ips(self):
-        self.instance_task.attach_public_ips(['fake-floatingip-id'])
-        mock_list_ports = (self.instance_task.neutron_client
-                           .list_ports)
-        mock_list_ports.assert_called_once_with(device_id='computeinst-id-1')
-
-        mock_update_floatingip = (self.instance_task.neutron_client
-                                  .update_floatingip)
-        mock_update_floatingip.assert_called_once_with(
-            'fake-floatingip-id',
-            {'floatingip': {'port_id': 'fake-port-id',
-                            'fixed_ip_address': '10.0.0.1'}})
-
     @patch.object(BaseInstance, 'update_db')
     def test_enable_as_master(self, mock_update_db):
         test_func = self.instance_task._guest.enable_as_master
@@ -991,27 +967,20 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
         self.instance_task.demote_replication_master()
         self.instance_task._guest.demote_replication_master.assert_any_call()
 
-    @patch.multiple(taskmanager_models.BuiltInstanceTasks,
-                    get_injected_files=Mock(return_value="the-files"))
-    def test_upgrade(self, *args):
-        pre_rebuild_server = self.instance_task.server
-        dsv = Mock(image_id='foo_image')
-        mock_volume = Mock(attachments=[{'device': '/dev/mock_dev'}])
-        with patch.object(self.instance_task._volume_client.volumes, "get",
-                          Mock(return_value=mock_volume)):
-            mock_server = Mock(status='ACTIVE')
-            with patch.object(self.instance_task._nova_client.servers,
-                              'get', Mock(return_value=mock_server)):
-                with patch.multiple(self.instance_task._guest,
-                                    pre_upgrade=Mock(return_value={}),
-                                    post_upgrade=Mock()):
-                    self.instance_task.upgrade(dsv)
+    @patch('trove.taskmanager.models.BuiltInstanceTasks.set_service_status')
+    @patch('trove.taskmanager.models.BuiltInstanceTasks.is_service_healthy')
+    @patch('trove.taskmanager.models.BuiltInstanceTasks.reset_task_status')
+    def test_upgrade(self, mock_resetstatus, mock_check, mock_setstatus):
+        dsv = MagicMock()
+        attrs = {'name': 'new_version'}
+        dsv.configure_mock(**attrs)
+        mock_check.return_value = True
+        self.instance_task._guest.pre_upgrade.return_value = {}
 
-                    self.instance_task._guest.pre_upgrade.assert_called_with()
-                    pre_rebuild_server.rebuild.assert_called_with(
-                        dsv.image_id, files="the-files")
-                    self.instance_task._guest.post_upgrade.assert_called_with(
-                        mock_volume.attachments[0])
+        self.instance_task.upgrade(dsv)
+
+        self.instance_task._guest.upgrade.assert_called_once_with(
+            {'datastore_version': 'new_version'})
 
     def test_fix_device_path(self):
         self.assertEqual("/dev/vdb", self.instance_task.
@@ -1030,7 +999,7 @@ class BackupTasksTest(trove_testtools.TestCase):
         self.backup.id = 'backup_id'
         self.backup.name = 'backup_test',
         self.backup.description = 'test desc'
-        self.backup.location = 'http://xxx/z_CLOUD/12e48.xbstream.gz'
+        self.backup.location = 'http://xxx/z_CLOUD/container/12e48.xbstream.gz'
         self.backup.instance_id = 'instance id'
         self.backup.created = 'yesterday'
         self.backup.updated = 'today'
@@ -1080,6 +1049,18 @@ class BackupTasksTest(trove_testtools.TestCase):
             "backup should be in DELETE_FAILED status"
         )
 
+    @patch('trove.common.clients.create_swift_client')
+    def test_delete_backup_delete_swift(self, mock_swift_client):
+        client_mock = MagicMock()
+        mock_swift_client.return_value = client_mock
+
+        taskmanager_models.BackupTasks.delete_backup(mock.ANY, self.backup.id)
+
+        client_mock.head_object.assert_called_once_with('container',
+                                                        '12e48.xbstream.gz')
+        client_mock.delete_object.assert_called_once_with('container',
+                                                          '12e48.xbstream.gz')
+
     def test_parse_manifest(self):
         manifest = 'container/prefix'
         cont, prefix = taskmanager_models.BackupTasks._parse_manifest(manifest)
